@@ -10,13 +10,15 @@ from functools import partial
 from logging import DEBUG
 
 # Qt imports
-from PySide2.QtCore import Qt, QSettings, QByteArray, QDir, QUrl, Signal
+from PySide2.QtCore import Qt, QSettings, QByteArray, QDir, QUrl, Signal, QSize
 from PySide2.QtWidgets import *
 from PySide2.QtGui import QIcon, QKeySequence, QDesktopServices
 
 
 # Custom imports
+from cutevariant import LOGGER
 from cutevariant.core import get_sql_connection, get_metadatas, command
+from cutevariant.core import sql
 from cutevariant.core.sql import get_database_file_name
 from cutevariant.core.writer import CsvWriter, PedWriter
 from cutevariant.gui import FIcon
@@ -33,9 +35,9 @@ from cutevariant.commons import (
 from cutevariant.gui.export import ExportDialogFactory, ExportDialog
 
 # Import plugins
-from cutevariant.gui import plugin
+from cutevariant.gui import plugin, plugin_form
 
-LOGGER = cm.logger()
+from cutevariant import LOGGER
 
 import copy
 
@@ -47,9 +49,9 @@ class StateData:
 
     """
 
-    def __init__(self, data=None):
-        self._data = data if data else {}
+    def __init__(self):
         self._changed = set()
+        self.reset()
 
     def __setitem__(self, key, value):
 
@@ -72,6 +74,13 @@ class StateData:
     def changed(self):
         return self._changed
 
+    def reset(self):
+        self._data = {
+            "fields": ["favorite", "classification", "chr", "pos", "ref", "alt"],
+            "source": "variants",
+            "filters": {},
+        }
+
 
 class MainWindow(QMainWindow):
     """Cutevariant mainwindow
@@ -99,27 +108,20 @@ class MainWindow(QMainWindow):
         self.app_settings = QSettings()
 
         # State variable of application changed by plugins
-        self._state_data = StateData(
-            {
-                "fields": ["favorite", "classification", "chr", "pos", "ref", "alt"],
-                "source": "variants",
-                "filters": {},
-            }
-        )
+        self._state_data = StateData()
 
         ## ===== GUI Setup =====
-
         self.setWindowTitle("Cutevariant")
         self.setWindowIcon(QIcon(DIR_ICONS + "app.png"))
         self.setWindowFlags(Qt.WindowContextHelpButtonHint | self.windowFlags())
 
-        # Setup menu bar
-        self.setup_menubar()
-
         # Setup ToolBar
         self.toolbar = self.addToolBar("maintoolbar")
         self.toolbar.setObjectName("maintoolbar")  # For window saveState
-        self.setup_toolbar()
+        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+
+        # Setup menu bar
+        self.setup_actions()
 
         # Setup Central widget
         self.central_tab = QTabWidget()
@@ -127,6 +129,7 @@ class MainWindow(QMainWindow):
         self.vsplit = QSplitter(Qt.Vertical)
         self.vsplit.addWidget(self.central_tab)
         self.vsplit.addWidget(self.footer_tab)
+        self.vsplit.setHandleWidth(5)
         self.setCentralWidget(self.vsplit)
         self.setTabPosition(Qt.AllDockWidgetAreas, QTabWidget.North)
         # registers plugins
@@ -206,7 +209,7 @@ class MainWindow(QMainWindow):
         action.setText(widget.windowTitle())
         self.view_menu.addAction(action)
 
-        self.toolbar.addAction(action)
+        # self.toolbar.addAction(action)
 
     def register_plugins(self):
         """Load all plugins to the window
@@ -214,7 +217,7 @@ class MainWindow(QMainWindow):
         See self.register_plugin
 
         """
-        LOGGER.info("MainWindow:: Registering plugins...")
+        LOGGER.info("Registering plugins...")
         # Get classes of plugins
         for extension in plugin.find_plugins():
             self.register_plugin(extension)
@@ -232,7 +235,6 @@ class MainWindow(QMainWindow):
         Args:
             extension (dict): Extension dict returned by `cutevariant.gui.plugin.find_plugins`
         """
-        LOGGER.debug("Extension: %s", extension)
 
         name = extension["name"]
         title = extension["title"]
@@ -307,6 +309,7 @@ class MainWindow(QMainWindow):
             sender (PluginWidget): from a plugin, you can pass "self" as argument
         """
 
+        plugin_to_refresh = []
         for plugin_obj in self.plugins.values():
             need_refresh = (
                 plugin_obj is not sender
@@ -316,14 +319,17 @@ class MainWindow(QMainWindow):
 
             if need_refresh:
                 try:
-                    plugin_obj.on_refresh()
-                    print(plugin_obj)
+                    plugin_to_refresh.append(plugin_obj)
+                    # plugin_obj.on_refresh()
+                    LOGGER.debug(f"refresh {plugin_obj.__class__}")
 
                 except Exception as e:
                     LOGGER.exception(e)
 
         # Clear state_changed set
         self._state_data.clear_changed()
+        for plugin in plugin_to_refresh:
+            plugin.on_refresh()
 
     def refresh_plugin(self, plugin_name: str):
         """Refresh a widget plugin identified by plugin_name
@@ -337,11 +343,12 @@ class MainWindow(QMainWindow):
             plugin_obj = self.plugins[plugin_name]
             plugin_obj.on_refresh()
 
-    def setup_menubar(self):
+    def setup_actions(self):
         """Menu bar setup: items and actions
 
         .. note:: Setup tools menu that could be dynamically augmented by plugins.
         """
+
         ## File Menu
         self.file_menu = self.menuBar().addMenu(self.tr("&File"))
         self.new_project_action = self.file_menu.addAction(
@@ -353,8 +360,17 @@ class MainWindow(QMainWindow):
             self.open_project,
             QKeySequence.Open,
         )
+
+        self.toolbar.addAction(self.new_project_action)
+        self.toolbar.addAction(self.open_project_action)
+        self.toolbar.addAction(
+            FIcon(0xF02D7), self.tr("Help"), QWhatsThis.enterWhatsThisMode
+        )
+        self.toolbar.addSeparator()
+
         ### Recent projects
         self.recent_files_menu = self.file_menu.addMenu(self.tr("Open recent"))
+
         self.setup_recent_menu()
 
         self.file_menu.addAction(QIcon(), self.tr("Export..."), self.on_export_pressed)
@@ -413,11 +429,23 @@ class MainWindow(QMainWindow):
         self.view_menu.addAction(self.tr("Reset widgets positions"), self.reset_ui)
         # Set toggle footer visibility action
         show_action = self.view_menu.addAction(
-            FIcon(0xF018D), self.tr("Show bottom toolbar")
+            FIcon(0xF018D), self.tr("Show VQL editor")
         )
         show_action.setCheckable(True)
+        self.toolbar.addAction(show_action)
         show_action.setChecked(True)
         show_action.toggled.connect(self.toggle_footer_visibility)
+
+        fullscreen_action = self.view_menu.addAction(
+            self.tr("Enter Full Screen"),
+        )
+
+        fullscreen_action.setShortcut(QKeySequence.FullScreen)
+        fullscreen_action.setCheckable(True)
+        fullscreen_action.toggled.connect(
+            lambda x: self.showFullScreen() if x else self.showNormal()
+        )
+
         self.view_menu.addSeparator()
 
         ## Tools
@@ -446,6 +474,11 @@ class MainWindow(QMainWindow):
         )
 
         self.help_menu.addSeparator()
+        # Setup developers menu
+        self.developers_menu = QMenu(self.tr("Developers..."))
+        self.setup_developers_menu()
+        self.help_menu.addMenu(self.developers_menu)
+
         self.help_menu.addAction(
             self.tr("About Qt..."), QApplication.instance().aboutQt
         )
@@ -455,20 +488,6 @@ class MainWindow(QMainWindow):
             self.about_cutevariant,
         )
 
-    def setup_toolbar(self):
-        """Top tool bar setup: items and actions
-
-        .. note:: Require selection_widget and some actions of Menubar
-        """
-        # Tool bar
-        self.toolbar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
-        self.toolbar.addAction(self.new_project_action)
-        self.toolbar.addAction(self.open_project_action)
-        self.toolbar.addAction(
-            FIcon(0xF02D7), self.tr("Help"), QWhatsThis.enterWhatsThisMode
-        )
-        self.toolbar.addSeparator()
-
     def open(self, filepath):
         """Open the given db/project file
 
@@ -530,7 +549,10 @@ class MainWindow(QMainWindow):
         """
         self.conn = conn
 
+        self._state_data.reset()
+
         # Clear memoization cache for count_cmd
+        sql.clear_lru_cache()
         # Clear State variable of application
         # store fields, source, filters, group_by, having data
 
@@ -589,7 +611,6 @@ class MainWindow(QMainWindow):
     def on_recent_project_clicked(self):
         """Slot to load a recent project"""
         action = self.sender()
-        LOGGER.debug(action.text())
         self.open(action.text())
 
     def new_project(self):
@@ -662,7 +683,12 @@ class MainWindow(QMainWindow):
     def show_settings(self):
         """Slot to show settings window"""
         widget = SettingsDialog(self)
-        widget.exec_()
+
+        if widget.exec_():
+            self.reload()
+
+    def reload(self):
+        self.open_database(self.conn)
 
     def show_dialog(self):
         """Show Plugin dialog box after a click on the tool menu"""
@@ -785,7 +811,7 @@ class MainWindow(QMainWindow):
         super().closeEvent(event)
 
     def save_session(self):
-        """ save plugin state into a json file """
+        """save plugin state into a json file"""
 
         filename, _ = QFileDialog.getSaveFileName(
             self,
@@ -870,7 +896,11 @@ class MainWindow(QMainWindow):
 
     def toggle_footer_visibility(self, visibility):
         """Toggle visibility of the bottom toolbar and all its plugins"""
-        self.footer_tab.setVisible(visibility)
+        # self.footer_tab.setVisible(visibility)
+        if visibility:
+            self.vsplit.setSizes([100, 0])
+        else:
+            self.vsplit.setSizes([200, 100])
 
     def on_export_pressed(self):
         """
@@ -957,6 +987,17 @@ class MainWindow(QMainWindow):
                 self.tr(f"Cannot save file to {os.path.basename(file_name)}"),
             )
 
+    def setup_developers_menu(self):
+        self.developers_menu.setIcon(FIcon(0xF1064))
+        self.create_plugin_action: QAction = self.developers_menu.addAction(
+            self.tr("Create new plugin")
+        )
+        self.create_plugin_action.setIcon(FIcon(0xF14D0))
+        # The resulting dialog is created and generates the plugin
+        self.create_plugin_action.triggered.connect(plugin_form.create_dialog_plugin)
+
+        return self.developers_menu
+
     # @Slot()
     # def on_query_model_changed(self):
     #     for name, _plugin in self.plugins.items():
