@@ -21,8 +21,7 @@ from PySide2.QtGui import (
     QFont,
     QFontMetrics,
     QSyntaxHighlighter,
-    QTextCharFormat
-
+    QTextCharFormat,
 )
 from PySide2.QtCore import (
     Qt,
@@ -36,11 +35,11 @@ from PySide2.QtCore import (
     QAbstractListModel,
     QRect,
     Signal,
-    QRegularExpression
-
+    QRegularExpression,
 )
 import sys
 
+import re
 
 
 class VqlSyntaxHighlighter(QSyntaxHighlighter):
@@ -154,14 +153,12 @@ class VqlSyntaxHighlighter(QSyntaxHighlighter):
                 self.setFormat(match.capturedStart(), match.capturedLength(), t_format)
 
 
-
-
 class CompleterModel(QAbstractListModel):
 
-    """Model used by Completer which contains keyword name, description, icon and color 
-    Use add_item to add new items . 
+    """Model used by Completer which contains keyword name, description, icon and color
+    Use add_item to add new items .
     Exemples:
-    
+
         model.beginResetModel()
         model.add_item("keyword", "description", QIcon(), "white")
         model.add_item("keyword", "description", QIcon(), "white")
@@ -175,42 +172,36 @@ class CompleterModel(QAbstractListModel):
 
         self._items = []
 
-
     def clear(self):
-        """Clear model
-        """
+        """Clear model"""
         self.beginResetModel()
         self._items.clear()
         self.endResetModel()
 
-
-    def add_item(self, name:str, description:str, icon = QIcon(), color = None):
-        """Add items 
+    def add_item(self, name: str, description: str, icon=QIcon(), color=None):
+        """Add items
 
         Todo:
             Create a signal with beginInsert
-        
+
         Args:
             name (str): keyword name to be inserted
             description (str): a description of the keyword
-            icon (TYPE, optional): the icon 
+            icon (TYPE, optional): the icon
             color (None, optional): the background color icon
         """
-        self._items.append({
-            "name": name,
-            "description": description, 
-            "icon": icon,
-            "color": color
-            })
+        self._items.append(
+            {"name": name, "description": description, "icon": icon, "color": color}
+        )
 
     def rowCount(self, parent=QModelIndex()) -> int:
         """Override from QAbstractListModel
-        
+
         Args:
             parent (QModelIndex, optional)
-        
+
         Returns:
-            int: row count 
+            int: row count
         """
         if parent == QModelIndex():
             return len(self._items)
@@ -218,13 +209,13 @@ class CompleterModel(QAbstractListModel):
 
     def data(self, index: QModelIndex, role: Qt.ItemDataRole):
         """Override from QAbstractListModel
-        
+
         Args:
             index (QModelIndex): index
-            role (Qt.ItemDataRole): role 
-        
+            role (Qt.ItemDataRole): role
+
         Returns:
-            Any: value 
+            Any: value
         """
         if not index.isValid():
             return None
@@ -246,12 +237,13 @@ class CompleterModel(QAbstractListModel):
 
 class CompleterDelegate(QStyledItemDelegate):
 
-    """CompleterDelegate is use by the completer to draw nicely icon and elements of the completer
-    """
+    """CompleterDelegate is use by the completer to draw nicely icon and elements of the completer"""
 
-    def paint(self, painter:QPainter, option: QStyleOptionViewItem , index: QModelIndex):
+    def paint(
+        self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex
+    ):
         """Paint a cell according index and option
-        
+
         Args:
             painter (QPainter)
             option (QStyleOptionViewItem)
@@ -271,7 +263,7 @@ class CompleterDelegate(QStyledItemDelegate):
         icon = index.data(Qt.DecorationRole)
         icon_color = index.data(Qt.BackgroundColorRole)
 
-        # draw icon background 
+        # draw icon background
         area = QRect(
             option.rect.x(), option.rect.y(), option.rect.height(), option.rect.height()
         )
@@ -279,12 +271,12 @@ class CompleterDelegate(QStyledItemDelegate):
         painter.setBrush(QColor(icon_color))
         painter.drawRect(area)
 
-        # Draw icon 
+        # Draw icon
         if icon:
             icon_area = area.adjusted(3, 3, -3, -3)
             painter.drawPixmap(icon_area, icon.pixmap(icon_area.size()))
 
-        # Draw text 
+        # Draw text
         text_rect = option.rect
         text_rect.setLeft(option.rect.height() + 3)
 
@@ -315,10 +307,10 @@ class Completer(QWidget):
         model (CompleterModel): the model
         proxy_model (QSortFilterProxyModel ): the proxy model used to filter model
         panel (QLabel): The description widget
-        view (QListView): the view 
-    
+        view (QListView): the view
+
     Signals:
-        activated (str): return the keyword selected 
+        activated (str): return the keyword selected
     """
 
     activated = Signal(str)
@@ -361,7 +353,7 @@ class Completer(QWidget):
         self.panel.setWordWrap(True)
         self.panel.setFrameShape(QFrame.StyledPanel)
 
-        # Create layout 
+        # Create layout
         vlayout = QHBoxLayout()
         vlayout.setContentsMargins(0, 0, 0, 0)
         vlayout.setSpacing(0)
@@ -370,21 +362,21 @@ class Completer(QWidget):
         self.setLayout(vlayout)
 
     def set_target(self, target):
-        """Set CodeEdit  
-        
+        """Set CodeEdit
+
         Args:
-            target (CodeEdit): The CodeEdit 
+            target (CodeEdit): The CodeEdit
         """
         self._target = target
         self.installEventFilter(self._target)
 
     def eventFilter(self, obj: QObject, event: QEvent) -> bool:
         """Filter event from CodeEdit and QListView
-        
+
         Args:
             obj (QObject): Description
             event (QEvent): Description
-        
+
         Returns:
             bool
         """
@@ -414,7 +406,7 @@ class Completer(QWidget):
                     event.ignore()
                     return True
 
-                # use tab to move down/up in the list 
+                # use tab to move down/up in the list
                 if event.key() == Qt.Key_Tab:
                     if current.row() < self.proxy_model.rowCount() - 1:
                         self.view.setCurrentIndex(
@@ -426,17 +418,17 @@ class Completer(QWidget):
                             self.proxy_model.index(current.row() - 1, 0)
                         )
 
-                # Route other key event to the target ! This make possible to write text when completer is visible 
+                # Route other key event to the target ! This make possible to write text when completer is visible
                 self._target.event(event)
 
         return super().eventFilter(obj, event)
 
     def complete(self, rect: QRect):
         """Show completer as popup
-        
+
         Args:
             rect (QRect): the area where to display the completer
-        
+
         """
         if self.proxy_model.rowCount() == 0:
             self.hide()
@@ -448,40 +440,46 @@ class Completer(QWidget):
             self.setFocus()
             if not self.isVisible():
                 width = 400
-                #height = self.view.sizeHintForRow(0) * self.proxy_model.rowCount() + 3
+                # height = self.view.sizeHintForRow(0) * self.proxy_model.rowCount() + 3
                 #  HACK.. TODO better !
-                #height = min(self._target.height() / 2, height)
+                # height = min(self._target.height() / 2, height)
 
-                #self.resize(width, height)
+                # self.resize(width, height)
                 self.adjustSize()
                 self.show()
 
     def set_completion_prefix(self, prefix: str):
-        """Set prefix and filter model 
-        
+        """Set prefix and filter model
+
         Args:
             prefix (str): A prefix keyword used to filter model
         """
         self.view.clearSelection()
-        self._completion_prefix = prefix
-        self.proxy_model.setFilterRegularExpression(QRegularExpression(f"^{prefix}.*", QRegularExpression.CaseInsensitiveOption))
+        self._completion_prefix = QRegularExpression.escape(prefix)
+
+        self.proxy_model.setFilterRegularExpression(
+            QRegularExpression(
+                f"^{ self._completion_prefix}.*",
+                QRegularExpression.CaseInsensitiveOption,
+            )
+        )
         if self.proxy_model.rowCount() > 0:
             self.select_row(0)
 
     def select_row(self, row: int):
         """Select a row in the model
-        
+
         Args:
             row (int): a row number
         """
         index = self.proxy_model.index(row, 0)
         self.view.selectionModel().setCurrentIndex(index, QItemSelectionModel.Select)
 
-    def completion_prefix(self)->str:
+    def completion_prefix(self) -> str:
         """getter of completion_prefix
-        
-        TODO: use getter / setter 
-        
+
+        TODO: use getter / setter
+
         Returns:
             str: Return the completion_prefix
         """
@@ -490,7 +488,7 @@ class Completer(QWidget):
     def hide(self):
         """Override from QWidget
 
-        Hide the completer 
+        Hide the completer
         """
         self.set_completion_prefix("")
         super().hide()
@@ -498,7 +496,7 @@ class Completer(QWidget):
     def _on_row_changed(self, current: QModelIndex, previous: QModelIndex):
         """Slot received when user select a new item in the list.
         This is used to update the panel
-        
+
         Args:
             current (QModelIndex): the selection index
             previous (QModelIndex): UNUSED
@@ -510,7 +508,7 @@ class Completer(QWidget):
 class CodeEdit(QTextEdit):
     """
     A QTextEdit code editor with a custom completer ( not QCompleter) and a VQL syntax Highlighter.
-    To make completer available, you should fill it . 
+    To make completer available, you should fill it .
 
     Examples:
 
@@ -531,9 +529,8 @@ class CodeEdit(QTextEdit):
         self.completer = Completer()
         self.completer.set_target(self)
 
-        self.syntax =VqlSyntaxHighlighter(self.document())
-
-
+        self.syntax = VqlSyntaxHighlighter(self.document())
+        self.setAcceptRichText(False)
         font = QFont("monospace")
         self.setFont(font)
 
@@ -565,7 +562,7 @@ class CodeEdit(QTextEdit):
 
         super().keyPressEvent(event)
 
-        #Skip modifier event 
+        # Skip modifier event
         has_modifier = event.modifiers() == Qt.ControlModifier
         if has_modifier:
             return
@@ -580,39 +577,46 @@ class CodeEdit(QTextEdit):
             self.completer.set_completion_prefix(word)
             self.completer.complete(rect)
 
-    def text_under_cursor(self)->str:
+    def text_under_cursor(self) -> str:
         """return text under cursor
-        
+
         Returns:
             str
         """
         tc = self.textCursor()
-        tc.select(QTextCursor.WordUnderCursor)
-        return tc.selectedText()
+        tc.setPosition(0, QTextCursor.KeepAnchor)
+        match = re.findall(r"([\w\.]+)$", tc.selectedText())
+        return match[0] if match else ""
 
-    def insert_completion(self, completion:str):
+    def insert_completion(self, completion: str):
         """Replace current word by completion
-        
+
         Args:
             completion (str)
         """
         tc = self.textCursor()
         extra = len(self.completer.completion_prefix())
+        text_under_cursor = self.text_under_cursor()
+        tc.movePosition(
+            QTextCursor.Left, QTextCursor.KeepAnchor, len(text_under_cursor)
+        )
 
-        tc.select(QTextCursor.WordUnderCursor)
         tc.removeSelectedText()
-        #tc.movePosition(QTextCursor.Left)
-        #tc.movePosition(QTextCursor.StartOfWord)
+        # tc.movePosition(QTextCursor.Left)
+        # tc.movePosition(QTextCursor.StartOfWord)
         tc.insertText(completion + " ")
 
 
-
-
 if __name__ == "__main__":
     app = QApplication(sys.argv)
 
-
     w = CodeEdit()
+    w.completer.model.beginResetModel()
+    w.completer.model.add_item("keyword", "description", QIcon(), "white")
+    w.completer.model.add_item("keyword", "description", QIcon(), "white")
+    w.completer.model.add_item("keyword", "description", QIcon(), "white")
+    w.completer.model.endResetModel()
+
     w.show()
 
     app.exec_()
